#!/usr/bin/perl
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted
# material is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Milan Zazrivec

# Description: standard users login smoke test
# Concept is:
#     1) get list of valid shells from /etc/shells
#     2) scan /etc/passwd for real users (for RHEL/FC that means UID >= 500):
#        a) how many users with UID == 0 do we have? -> $uid_zero
#        b) does every user has an existing homedir?
#        c) does user's shell exist?
#        d) is user's shell listed in /etc/shells?
#        e) is user able to login?

use strict;
use warnings FATAL => 'all';

print "Running users login test.\n\n".
      "Test will scan /etc/passwd for real users (in FC/RHEL UID >= 500)\n".
      "and perform standard login test. Program will check if every user has\n".
      "an existing home directory and a valid shell (listed in /etc/shells).".
      "\n\n";

my $errors = 0;
my $uid_limit = 500;
my $gid_limit = 500;
my $uid_zero = 0;
my @uid_zero_users;
my %shells;
my ($user, $uid, $gid, $homedir, $shell);

# get list of valid shells -> %shells
die "$!\n" unless open FILE, '< /etc/shells';
while (<FILE>)
{
    chomp;
    $shells{ $_ } = $.;
}
close FILE;

# scan /etc/passwd
die "$!\n" unless open FILE, '< /etc/passwd';
print "Scanning /etc/passwd...\n";
while (<FILE>)
{
    chomp;
    # does the line look OK?
    if (not /^(.*):.+:(\d+):(\d+):(.*):(.*):(.*)$/)
    {   # very unlike to find malformed line
        print "/etc/passwd: line $. malformed: $_\n";
        next;
    }
    $user = $1;
    # we'll have a separate list of users with $uid == 0
    ++$uid_zero and push @uid_zero_users, $user if ($uid = $2) == 0;
    # we'll try to find 'real' users ($uid > $uid_limit)
    next if (not $uid >= $uid_limit || $uid == 0);
    print "\nFound user $user:\n";
    print "- checking user's home directory: ";
    # does the homedir for $user exist?
    if (($uid >= $uid_limit or $uid == 0) and not -d ($homedir = $5))
    {
        print "directory $homedir does not exist!\n";
        ++$errors;
    }
    else
    {
        print "OK\n";
    }
    # does the shell for $user exist?
    print "- checking existence of user's shell: ";
    if (not -f ($shell = $6))
    {
        ++$errors;
        print "shell $shell does not exist!\n";
        next;
    }
    else
    {
        print "OK\n";
    }
    # is user's shell a valid shell?
    print "- checking presence of $shell in /etc/shells: ";
    if (not exists $shells { $shell })
    {
        ++$errors;
        print "not listed in /etc/shells!\n";
        next;
    }
    else
    {
        print "OK\n";
    }

    # is user's shell executable?
    print "- checking $shell for executable flag: ";
    if ("$shell" and ((not -x $shell) or (not -X $shell)))
    {
        ++$errors;
        print "FAILED\n";
    }
    else
    {
        print "OK\n";
    }
    if ($shell =~ /(.*nologin)|(.*false)/)
    {
        next;
    }
    # basic login test
    else
    {
        print "- performing simple login test for $user: ";
        if (system ("su -l $user -c /bin/true >& /dev/null"))
        {
            ++$errors;
            print "FAILED\n";
        }
	else
        {
            print "OK\n";
        }
    }
}
close FILE;

if ($uid_zero > 1)
{
    print "\nFound $uid_zero users with UID 0: @uid_zero_users\n";
}

if ($errors)
{
    print "\nTest complete: result=FAIL\n";
}
else
{
    print "\nTest complete: result=PASS\n";
}

exit $errors;
